@@ -8,6 +8,28 @@ from ..models import LocalizedRecurrence, LocalizedRecurrenceQuerySet
 from ..models import _replace_with_offset, _update_schedule
 
 
+class LocalizedRecurrenceUpdateTest(TestCase):
+    """
+    Tests calling 'update' on a localized recurrence.
+    """
+    def test_update_creation(self):
+        lr = LocalizedRecurrence()
+        lr.update()
+        self.assertIsNotNone(lr.id)
+
+    def test_update_timezone(self):
+        lr = G(LocalizedRecurrence)
+        lr.update(timezone='US/Eastern')
+        lr = LocalizedRecurrence.objects.get(id=lr.id)
+        self.assertEqual(lr.timezone, pytz.timezone('US/Eastern'))
+
+    def test_update_offset(self):
+        lr = G(LocalizedRecurrence)
+        lr.update(offset=timedelta(seconds=1))
+        lr = LocalizedRecurrence.objects.get(id=lr.id)
+        self.assertEqual(lr.offset, timedelta(seconds=1))
+
+
 class LocalizedRecurrenceQuerySetTest(TestCase):
     """Simple test to ensure the custom query set is being used.
     """
@@ -58,6 +80,13 @@ class LocalizedRecurrenceTest(TestCase):
         lr = LocalizedRecurrence.objects.first()
         self.assertTrue(isinstance(lr.offset, timedelta))
 
+    def test_string_representation(self):
+        lr = LocalizedRecurrence.objects.first()
+        self.assertEqual(
+            str(lr),
+            'ID: {0}, Interval: {1}, Next Scheduled: {2}'.format(lr.id, lr.interval, lr.next_scheduled),
+        )
+
 
 class LocalizedRecurrenceUpdateScheduleTest(TestCase):
     def setUp(self):
@@ -94,7 +123,8 @@ class LocalizedRecurrenceUtcOfNextScheduleTest(TestCase):
             timezone=pytz.timezone('Asia/Hong_Kong'))
 
     def test_basic_works(self):
-        """Test a simple case of utc_of_next_schedule.
+        """
+        Test a simple case of utc_of_next_schedule.
 
         - The given recurrence is scheduled daily for Eastern Time at noon.
         - The given current date in UTC is 2013/1/15::17:05:22
@@ -105,6 +135,13 @@ class LocalizedRecurrenceUtcOfNextScheduleTest(TestCase):
         schedule_out = self.lr_day.utc_of_next_schedule(current_time)
         self.assertEqual(schedule_out, expected_next_schedule)
 
+    def test_dst_cross_monthly(self):
+        """The case when a monthly recurrence goes past daylight savings time"""
+        self.lr_month.offset = timedelta(hours=0)
+        self.lr_month.previous_scheduled = datetime(2015, 2, 1, 6)
+        scheduled_out = self.lr_month.utc_of_next_schedule(datetime(2015, 3, 31))
+        self.assertEqual(scheduled_out, datetime(2015, 4, 1, 5))
+
     def test_before_midnight(self):
         """The case when the scheduled and current time cross midnight.
 
@@ -120,7 +157,8 @@ class LocalizedRecurrenceUtcOfNextScheduleTest(TestCase):
         self.assertEqual(schedule_out, expected_next_schedule)
 
     def test_after_midnight(self):
-        """The case when the scheduled and current time are after midnight.
+        """
+        The case when the scheduled and current time are after midnight.
 
         - The given recurrence is scheduled daily for Eastern Time at 12:01 AM.
         - The given current date in UTC is 2013/1/15::05:05:22 -> 12:05AM EST
@@ -129,48 +167,139 @@ class LocalizedRecurrenceUtcOfNextScheduleTest(TestCase):
         self.lr_day.offset = timedelta(minutes=1)
         self.lr_day.save()
         current_time = datetime(2013, 1, 15, 5, 5, 22)
-        expected_next_schedule = datetime(2013, 1, 16, 5, 01)
+        expected_next_schedule = datetime(2013, 1, 16, 5, 1)
         schedule_out = self.lr_day.utc_of_next_schedule(current_time)
         self.assertEqual(schedule_out, expected_next_schedule)
 
-    def test_week_update(self):
-        """Weekly Recurrences should work as expected.
+    def test_week_update_full_week(self):
+        """
+        Weekly Recurrences should be able to add a full week.
 
-        - Friday August 8th at 10:34 PM UTC is 5:34 PM CDT.
-        - Scheduled for weekly, Friday at 5:30
-        - Expect next schedule to be Friday August 15th at 10:30 PM UTC
+        - Thursday August 8th at 10:34 PM UTC is 5:34 PM CDT.
+        - Scheduled for weekly, Thursday at 5:30
+        - Expect next schedule to be Thursday August 15th at 10:30 PM UTC
         """
         current_time = datetime(2013, 8, 8, 22, 34)
         expected_next_schedule = datetime(2013, 8, 15, 22, 30)
         schedule_out = self.lr_week.utc_of_next_schedule(current_time)
         self.assertEqual(schedule_out, expected_next_schedule)
 
-    def test_month_update(self):
-        """Monthly Recurrences should work as expected.
+    def test_weekly_update_current_week(self):
+        """
+        Weekly Recurrences should be able to work in the current week.
 
-        - Friday August 23rd at 12:34 AM UTC is Friday August 22nd at 7:34 PM CDT.
-        - Scheduled for Monthly, on the 8th day at 7:15.10 PM CDT
-        - Expect next schedule to be September 9 at 12:15.10 AM UTC
+        - Tuesday August 6th at 10:34 PM UTC is 5:34 PM CDT.
+        - Scheduled for weekly, Thursday at 5:30
+        - Expect next schedule to be Thursday August 8th at 10:30 PM UTC
+        """
+        current_time = datetime(2013, 8, 6, 22, 34)
+        expected_next_schedule = datetime(2013, 8, 8, 22, 30)
+        schedule_out = self.lr_week.utc_of_next_schedule(current_time)
+        self.assertEqual(schedule_out, expected_next_schedule)
+
+    def test_month_update_full_month(self):
+        """
+        Monthly Recurrences should work as expected moving forward a full month.
+
+        - Friday August 23rd at 12:34 AM UTC is Thursday August 22nd at 7:34 PM CDT.
+        - Scheduled for Monthly, on the 21st day at 7:15.10 PM
+        - Expect next schedule to be September 23 at 12:15.10 AM UTC
         """
         current_time = datetime(2013, 8, 23, 0, 34, 55)
         expected_next_schedule = datetime(2013, 9, 23, 0, 15, 10)
         schedule_out = self.lr_month.utc_of_next_schedule(current_time)
         self.assertEqual(schedule_out, expected_next_schedule)
 
-    def test_quarterly(self):
-        """Quarterly Recurrences should work as expected.
+    def test_month_update_current_month(self):
+        """
+        Monthly Recurrences should work as expected moving forward in the
+        current month.
+
+        - Friday August 16th at 12:34 AM UTC is Thursday August 15th at 7:34 PM CDT.
+        - Scheduled for Monthly, on the 21st day at 7:15.10 PM
+        - Expect next schedule to be August 23 at 12:15.10 AM UTC
+
+        """
+        current_time = datetime(2013, 8, 16, 0, 34, 55)
+        expected_next_schedule = datetime(2013, 8, 23, 0, 15, 10)
+        schedule_out = self.lr_month.utc_of_next_schedule(current_time)
+        self.assertEqual(schedule_out, expected_next_schedule)
+
+    def test_month_year_end_update(self):
+        """
+        Monthly Recurrences should transition over years correctly
+
+        - Monday December 23 at 2:34 AM UTC is Sunday August 22nd at 9:34 PM CDT.
+        - Scheduled for Monthly, on the 21st day at 7:15.10 PM CDT
+        - Expect next schedule to be January 23 at 1:15.10 AM UCT
+        """
+        current_time = datetime(2013, 12, 23, 2, 34, 55)
+        expected_next_schedule = datetime(2014, 1, 23, 1, 15, 10)
+        schedule_out = self.lr_month.utc_of_next_schedule(current_time)
+        self.assertEqual(schedule_out, expected_next_schedule)
+
+    def test_quarterly_full_quarter(self):
+        """
+        Quarterly Recurrences should be able to update a full quarter.
 
         - June 23rd at 12:34 AM UTC is June 22nd at 10:34 PM HKT.
         - Scheduled for Quarterly, on the 68th day at 4:30 PM HKT
-        - Expect next schedule to be September 8th at 8:30 AM UTC
+        - Expect next schedule to be September 7th at 8:30 AM UTC
         """
         current_time = datetime(2013, 6, 23, 0, 34, 55)
-        expected_next_schedule = datetime(2013, 9, 8, 8, 30)
+        expected_next_schedule = datetime(2013, 9, 7, 8, 30)
         schedule_out = self.lr_quarter.utc_of_next_schedule(current_time)
+        self.lr_quarter = G(
+            LocalizedRecurrence,
+            interval='QUARTER',
+            offset=timedelta(days=68, hours=16, minutes=30),
+            timezone=pytz.timezone('Asia/Hong_Kong')
+        )
+        self.assertEqual(schedule_out, expected_next_schedule)
+
+    def test_quarterly_current_quarter(self):
+        """
+        Quarterly Recurrences should be able to update in the current quarter.
+
+        - April 23rd at 12:34 AM UTC is April 22nd at 10:34 PM HKT.
+        - Scheduled for Quarterly, on the 68th day at 4:30 PM HKT
+        - Expect next schedule to be June 8th at 8:30 AM UTC
+        """
+        current_time = datetime(2013, 4, 23, 0, 34, 55)
+        expected_next_schedule = datetime(2013, 6, 8, 8, 30)
+        schedule_out = self.lr_quarter.utc_of_next_schedule(current_time)
+
+        self.lr_quarter = G(
+            LocalizedRecurrence,
+            interval='QUARTER',
+            offset=timedelta(days=68, hours=16, minutes=30),
+            timezone=pytz.timezone('Asia/Hong_Kong')
+        )
+
+        self.assertEqual(schedule_out, expected_next_schedule)
+
+    def test_quarterly_end_year(self):
+        """
+        Quarterly Recurrences should be able to update through year end.
+
+        - December 23rd at 12:34 AM UTC is April 22nd at 10:34 PM HKT.
+        - Scheduled for Quarterly, on the 68th day at 4:30 PM HKT
+        - Expect next schedule to be March 10th at 8:30 AM UTC
+        """
+        current_time = datetime(2013, 12, 23, 0, 34, 55)
+        expected_next_schedule = datetime(2014, 3, 10, 8, 30)
+        schedule_out = self.lr_quarter.utc_of_next_schedule(current_time)
+        self.lr_quarter = G(
+            LocalizedRecurrence,
+            interval='QUARTER',
+            offset=timedelta(days=68, hours=16, minutes=30),
+            timezone=pytz.timezone('Asia/Hong_Kong')
+        )
         self.assertEqual(schedule_out, expected_next_schedule)
 
     def test_yearly(self):
-        """Yearly Recurrences should work as expected.
+        """
+        Yearly Recurrences should work as expected.
 
         - June 23rd at 12:34 AM UTC is June 22nd at 10:34 PM HKT.
         - Scheduled for Yearly, on the 31st day at 4:30 PM HKT
@@ -182,7 +311,8 @@ class LocalizedRecurrenceUtcOfNextScheduleTest(TestCase):
         self.assertEqual(schedule_out, expected_next_schedule)
 
     def test_into_dst_boundary(self):
-        """Recurrences happen at the correct local time after going into DST.
+        """
+        Recurrences happen at the correct local time after going into DST.
 
         Going into daylight savings time should mean the UTC time is
         an hour less on the next recurrence.
@@ -198,7 +328,8 @@ class LocalizedRecurrenceUtcOfNextScheduleTest(TestCase):
         self.assertEqual(schedule_out, expected_next_schedule)
 
     def test_out_of_dst_boundary(self):
-        """Recurrences at the correct local time after going out of UTC.
+        """
+        Recurrences at the correct local time after going out of UTC.
 
         Going into daylight savings time should mean the UTC time is
         an hour greater on the next recurrence.
@@ -214,7 +345,8 @@ class LocalizedRecurrenceUtcOfNextScheduleTest(TestCase):
         self.assertEqual(schedule_out, expected_next_schedule)
 
     def test_utc_plus(self):
-        """Test a timezone that is UTC + 2.
+        """
+        Test a timezone that is UTC + 2.
 
         - Europe/Berlin DST is UTC + 2
         """
@@ -247,7 +379,8 @@ class UpdateScheduleTest(TestCase):
 
 class ReplaceWithOffsetTest(TestCase):
     def test_day(self):
-        """_replace_with_offset works as expected with a 'DAY' interval.
+        """
+        _replace_with_offset works as expected with a 'DAY' interval.
         """
         dt_in = datetime(2013, 1, 20, 12, 45, 48)
         td_in = timedelta(hours=3, minutes=3, seconds=3)
@@ -257,7 +390,8 @@ class ReplaceWithOffsetTest(TestCase):
         self.assertEqual(dt_out, dt_expected)
 
     def test_week(self):
-        """_replace_with_offset works as expected with a 'WEEK' interval.
+        """
+        _replace_with_offset works as expected with a 'WEEK' interval.
         """
         dt_in = datetime(2013, 1, 20, 12, 45, 48)
         td_in = timedelta(days=4, hours=3, minutes=3, seconds=3)
@@ -267,7 +401,8 @@ class ReplaceWithOffsetTest(TestCase):
         self.assertEqual(dt_out, dt_expected)
 
     def test_week_on_month_boundary(self):
-        """_replace_with_offset using interval 'WEEK' should roll over months
+        """
+        _replace_with_offset using interval 'WEEK' should roll over months
         correctly.
         """
         dt_in = datetime(2013, 7, 30, 12, 45, 48)
@@ -278,7 +413,8 @@ class ReplaceWithOffsetTest(TestCase):
         self.assertEqual(dt_out, dt_expected)
 
     def test_month(self):
-        """_replace_with_offset works as expected with a 'MONTH' interval.
+        """
+        _replace_with_offset works as expected with a 'MONTH' interval.
         """
         dt_in = datetime(2013, 1, 20, 12, 45, 48)
         td_in = timedelta(days=15, hours=3, minutes=3, seconds=3)
@@ -288,12 +424,64 @@ class ReplaceWithOffsetTest(TestCase):
         self.assertEqual(dt_out, dt_expected)
 
     def test_last_day_of_month(self):
-        dt_in = datetime(2013, 2, 20, 12, 45, 48)
-        td_in = timedelta(days=31, hours=3, minutes=3, seconds=3)
-        interval_in = 'MONTH'
-        dt_expected = datetime(2013, 2, 28, 3, 3, 3)
-        dt_out = _replace_with_offset(dt_in, td_in, interval_in)
-        self.assertEqual(dt_out, dt_expected)
+        """
+        Check dates for a full year where the utc time is the first and the time zone is the previous day
+        """
+        time_delta = timedelta(days=30, hours=23, minutes=3, seconds=3)
+        dt_start = datetime(2013, 2, 20, 23, 45, 48)
+        interval_name = 'MONTH'
+        timezone_name = 'US/Central'
+        recurrence = LocalizedRecurrence.objects.create(
+            interval=interval_name,
+            offset=time_delta,
+            timezone=timezone_name,
+            next_scheduled=dt_start,
+        )
+
+        # Check a full year of dates. The first next recurrence should be the month after it starts because
+        # The start time is weird because it should be set correctly to begin with. Setting it to 2-20 should not
+        # be happening. The app should initially set it to the correct first fire date
+        self.assertEqual(recurrence.next_scheduled, datetime(2013, 2, 20, 23, 45, 48))
+        recurrence.update_schedule(recurrence.next_scheduled)
+
+        self.assertEqual(recurrence.next_scheduled, datetime(2013, 4, 1, 4, 3, 3))
+        recurrence.update_schedule(recurrence.next_scheduled)
+
+        self.assertEqual(recurrence.next_scheduled, datetime(2013, 5, 1, 4, 3, 3))
+        recurrence.update_schedule(recurrence.next_scheduled)
+
+        self.assertEqual(recurrence.next_scheduled, datetime(2013, 6, 1, 4, 3, 3))
+        recurrence.update_schedule(recurrence.next_scheduled)
+
+        self.assertEqual(recurrence.next_scheduled, datetime(2013, 7, 1, 4, 3, 3))
+        recurrence.update_schedule(recurrence.next_scheduled)
+
+        self.assertEqual(recurrence.next_scheduled, datetime(2013, 8, 1, 4, 3, 3))
+        recurrence.update_schedule(recurrence.next_scheduled)
+
+        self.assertEqual(recurrence.next_scheduled, datetime(2013, 9, 1, 4, 3, 3))
+        recurrence.update_schedule(recurrence.next_scheduled)
+
+        self.assertEqual(recurrence.next_scheduled, datetime(2013, 10, 1, 4, 3, 3))
+        recurrence.update_schedule(recurrence.next_scheduled)
+
+        self.assertEqual(recurrence.next_scheduled, datetime(2013, 11, 1, 4, 3, 3))
+        recurrence.update_schedule(recurrence.next_scheduled)
+
+        self.assertEqual(recurrence.next_scheduled, datetime(2013, 12, 1, 5, 3, 3))
+        recurrence.update_schedule(recurrence.next_scheduled)
+
+        self.assertEqual(recurrence.next_scheduled, datetime(2014, 1, 1, 5, 3, 3))
+        recurrence.update_schedule(recurrence.next_scheduled)
+
+        self.assertEqual(recurrence.next_scheduled, datetime(2014, 2, 1, 5, 3, 3))
+        recurrence.update_schedule(recurrence.next_scheduled)
+
+        self.assertEqual(recurrence.next_scheduled, datetime(2014, 3, 1, 5, 3, 3))
+        recurrence.update_schedule(recurrence.next_scheduled)
+
+        self.assertEqual(recurrence.next_scheduled, datetime(2014, 4, 1, 4, 3, 3))
+        recurrence.update_schedule(recurrence.next_scheduled)
 
     def test_quarter(self):
         dt_in = datetime(2013, 4, 20, 12, 45, 48)
@@ -352,7 +540,8 @@ class ReplaceWithOffsetTest(TestCase):
         self.assertEqual(dt_expected, dt_out)
 
     def test_bad_interval(self):
-        """A missformed interval should raise a value error
+        """
+        A missformed interval should raise a value error
         """
         dt_in = datetime(2013, 1, 20, 12, 45, 48)
         td_in = timedelta(days=15, hours=3, minutes=3, seconds=3)
